# -*-: coding utf-8 -*-
""" Helper methods for OS related tasks. """

from getpass import getpass
import os
import re
import shlex
import subprocess
import urllib2

from snipsskillscore.logging import log, log_success, log_error


def cmd_exists(cmd):
    """ Check if a command exists.

    :param cmd: the command to look for.
    :return: true if the command exists, false otherwise.
    """
    return subprocess.call("type " + cmd, shell=True,
                           stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0


def is_raspi_os():
    """ Check if the current system is Raspberry.

    :return: true if the current system is Raspberry.
    """
    return 'arm' in " ".join(os.uname())


def create_dir(dir_name):
    """ Create directory in the current working directory, if it does
        not exist already.

    :param dir_name: the name of the directory.
    """
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)


def create_dir_verbose(dir_name, indentation_level):
    log("\tcreating: {} {}".format(
        indentation_level * "+++",
        dir_name
    ))
    create_dir(dir_name)


def write_text_file(output_file_path, text):
    with open(output_file_path, "w") as output_file:
        output_file.write(text)


def write_text_file_verbose(output_file_path, text, indentation_level):
    write_text_file(output_file_path, text)

    log("\tcreating: {} {}".format(
        indentation_level * "+++",
        output_file_path
    ))


def execute_command(command, silent=False):
    """ Execute a shell command.

    :param command: the command to execute.
    :param silent: if True, do not output anything to terminal.
    """
    if silent:
        stdout = open(os.devnull, 'w')
    else:
        stdout = subprocess.PIPE
    subprocess.Popen(command.split(), stdout=stdout).communicate()


def pipe_commands(first_command, second_command, silent):
    """ Execute piped commands: `first_command | second_command`.

    :param first_command: the first command to execute.
    :param second_command: the second command to execute.
    """
    process1 = subprocess.Popen(first_command.split(), stdout=subprocess.PIPE)
    if silent:
        FNULL = open(os.devnull, 'w')
        process2 = subprocess.Popen(
            second_command.split(), stdin=process1.stdout, stdout=FNULL)
    else:
        process2 = subprocess.Popen(
            second_command.split(), stdin=process1.stdout)
    process1.stdout.close()
    process2.communicate()


def remove_file(file_path):
    """ Delete a file.

    :param file_path: the path to the file.
    """
    try:
        os.remove(file_path)
    except OSError:
        pass


def download_file(url, output_file):
    """ Download a file.

    :param url: the remote location of the file.
    :param output_file: the file to write to.
    """
    downloaded_file = urllib2.urlopen(url)
    with open(output_file, 'wb') as output:
        output.write(downloaded_file.read())


def ask_yes_no(question):
    """ Ask a yes/no question in the prompt.

    :param question: the question to ask.
    :return: true if the user answered yes (or empty), false otherwise
    """
    answer = raw_input("{} [Y/n] ".format(question))
    if answer is not None and answer.strip() != "" and answer.lower() != "y":
        return False
    return True


def ask_for_input(question, default_value=None):
    if default_value and len(default_value) > 0:
        answer = raw_input("{} [{}] ".format(question, default_value))
        if len(answer) == 0:  # The user hit enter.
            answer = default_value
    else:
        answer = raw_input(question)

    if answer is not None and answer.strip() != "":
        return answer
    else:
        return None


def ask_for_password(question):
    answer = getpass("{} ".format(question))
    if answer is not None and answer.strip() != "":
        return answer
    else:
        return None


def which(command):
    """ Get full path for an executable.

    :param command: the executable command, e.g. 'node'.
    :return: the full path for the command, e.g. '/usr/local/bin/node'.
    """
    try:
        return subprocess.check_output(
            ['which', command]).strip()
    except subprocess.CalledProcessError:
        return None


def reboot():
    """ Reboot the device."""
    execute_command("sudo reboot")


def get_os_name():
    os_release = subprocess.check_output(['cat', '/etc/os-release'])
    for line in os_release.splitlines():
        if line.startswith("PRETTY_NAME="):
            split = line.split("=")
            if len(split) > 1:
                os_version = split[1]
                return os_version.replace("\"", "")
    return None


def get_revision():
    process1 = subprocess.Popen('cat /proc/cpuinfo'.split(), stdout=subprocess.PIPE)
    process2 = subprocess.Popen('grep Revision'.split(), stdin=process1.stdout, stdout=subprocess.PIPE)
    process3 = subprocess.Popen(['awk', '{print $3}'], stdin=process2.stdout)
    process1.stdout.close()
    process2.stdout.close()
    return process3.communicate()


def get_sysinfo():
    return {
        "os_name": get_os_name()
    }


def get_command_output(command_array):
    return subprocess.check_output(command_array)


def get_user_email_git():
    if cmd_exists("git"):
        command = "git config user.email"
        output = get_command_output(command.split())
        if output is not None and len(output) > 0:
            return output.strip()
        return None
    else:
        return None


def email_is_valid(email):
    return True if re.match(r"[^@]+@[^@]+\.[^@]+", email) else False



# -*-: coding utf-8 -*-
"""The daemon command."""
# pylint: disable=too-few-public-methods,import-error

import os
import subprocess
import time
import threading

from sys import path

from snipsskillscore.logging import log, log_success, log_warning, log_error
from ..utils.rpc_daemon import RPCDaemon

from .base import Base


class RPCRun(Base):
    """TheRPC run command."""

    # pylint: disable=undefined-variable,exec-used,eval-used
    def run(self):
        """ Command runner. """

        snipsfile_path = self.options['--snipsfile']
        if snipsfile_path is None or len(snipsfile_path) == 0:
            snipsfile_path = SNIPSFILE

        try:
            self.snipsfile = Snipsfile(snipsfile_path)
        except SnipsfileNotFoundError:
            log_error("Snipsfile not found. Please create one.")
            return
        except SnipsfileParseException as err:
            log_error(err)
            return

        daemon = RPCDaemon(self.snipsfile.mqtt_hostname,
                        self.snipsfile.mqtt_port,
                        self.snipsfile.logging)

        log("Starting the Snips Daemon server.")
        daemon.start()



# -*-: coding utf-8 -*-
""" Utilities for managing the Snips SDK. """

import os
import subprocess

from .os_helpers import cmd_exists, is_raspi_os, ask_yes_no

from snipsskillscore.logging import log, log_success, log_warning, log_error

try:
    from subprocess import DEVNULL
except ImportError:
    DEVNULL = open(os.devnull, 'wb')

# pylint: disable=too-few-public-methods


SNIPS_INSTALL_COMMAND = "curl https://install.snips.ai -sSf"
SNIPS_INSTALL_ASSISTANT_COMMAND = "snips-install-assistant {}"


class SnipsUnsupportedPlatform(Exception):
    """ Unsupported platform exception class. """
    pass


class SnipsNotFound(Exception):
    """ Snips command not found exception class. """
    pass


class SnipsRuntimeFailure(Exception):
    """ Snips runtime failure exception class. """
    pass


class SnipsInstallationFailure(Exception):
    """ Snips installation failure exception class. """
    pass


class Snips:
    """ Utilities for managing the Snips SDK. """

    @staticmethod
    def install():
        """ Install the Snips SDK. """
        if Snips.is_installed():
            return

        if ask_yes_no("Would you like to install the Snips SDK?") == False:
            return

        log("Installing the Snips SDK.")

        if not is_raspi_os():
            raise SnipsUnsupportedPlatform()

        curl_command = subprocess.Popen(
            SNIPS_INSTALL_COMMAND.split(), stdout=subprocess.PIPE)
        sh_command = subprocess.Popen("sh", stdin=curl_command.stdout)
        output, error = sh_command.communicate()
        rc = sh_command.returncode

        if (rc > 0):
            raise SnipsInstallationFailure(output)
        else:
            log_success("The Snips SDK was successfully installed.")

    @staticmethod
    def run():
        """ Run the Snips SDK. """
        if not Snips.is_installed():
            raise SnipsNotFound()

        try:
            subprocess.Popen(
                "snips", stdout=DEVNULL, stderr=subprocess.STDOUT)
        except OSError as e:
            raise SnipsRuntimeFailure(str(e))

    @staticmethod
    def is_installed():
        """ Check if the Snips SDK is installed. """
        return cmd_exists("snips")

    @staticmethod
    def load_assistant(assistant_zip_path):
        """ Load an assistant file for the Snips SDK.

        :param assistant_zip_path: The path to the assistant.zip file.
        """
        process = subprocess.Popen(
            SNIPS_INSTALL_ASSISTANT_COMMAND.format(assistant_zip_path).split(),
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output, error = process.communicate()
        return process.returncode



# -*-: coding utf-8 -*-
""" Downloader for Snips assistants. """

import os
import shutil
import subprocess

from .os_helpers import cmd_exists, is_raspi_os, execute_command, pipe_commands


def copy_asoundrc(filename):
    """ Copy asoundrc configuration to local path.

    :param filename: the name of the asoundrc configuration, as
                     present in the config folder.
    """
    this_dir, this_filename = os.path.split(__file__)
    asoundrc_path = os.path.join(this_dir, "../config/asoundrc", filename)
    destination = os.path.expanduser('~/.asoundrc')
    shutil.copy2(asoundrc_path, destination)

# pylint: disable=too-few-public-methods


class MicrophoneSetup:
    """ Downloader for Snips assistants. """

    @staticmethod
    def setup(microphone_config=None):
        """ Setup microphone.

        :param microphone_id: the microphone id, e.g. 'respeaker'.
        """
        if microphone_config is not None and microphone_config.identifier == 'respeaker':
            RespeakerMicrophoneSetup.setup(microphone_config.params)
        elif microphone_config is not None and microphone_config.identifier == 'jabra':
            JabraMicrophoneSetup.setup()
        else:
            DefaultMicrophoneSetup.setup()


class DefaultMicrophoneSetup:

    @staticmethod
    def setup(asoundrc_file="asoundrc.default"):
        if not is_raspi_os():
            return
        copy_asoundrc(asoundrc_file)


class JabraMicrophoneSetup:

    @staticmethod
    def setup():
        DefaultMicrophoneSetup.setup("asoundrc.jabra")


class RespeakerMicrophoneSetup:

    @staticmethod
    def setup(params):
        if not is_raspi_os():
            return

        execute_command("sudo rm -f /lib/udev/rules.d/50-rspk.rules")

        echo_command = ("echo ACTION==\"add\", SUBSYSTEMS==\"usb\", ATTRS{{idVendor}}==\"{}\", " +
                        "ATTRS{{idProduct}}==\"{}\", MODE=\"660\", GROUP=\"plugdev\"") \
            .format(params["vendor_id"], params["product_id"])
        tee_command = "sudo tee --append /lib/udev/rules.d/50-rspk.rules"
        pipe_commands(echo_command, tee_command, silent=True)

        execute_command("sudo adduser pi plugdev")
        execute_command("sudo udevadm control --reload")
        execute_command("sudo udevadm trigger")

        copy_asoundrc("asoundrc.respeaker")



# -*-: coding utf-8 -*-
"""The daemon command."""
# pylint: disable=too-few-public-methods,import-error

import os
import subprocess
import time
import threading

from sys import path

from snipsskillscore.logging import log, log_success, log_warning, log_error


class RPCInstall(Base):
    """The run command."""

    # pylint: disable=undefined-variable,exec-used,eval-used
    def run(self):
        """ Command runner. """
        
        # Setup systemd

        pass



# -*-: coding utf-8 -*-
import os
from jinja2 import Environment, PackageLoader
from snipsskillscore.logging import log, log_error

from .base import Base
from ..utils.os_helpers import create_dir_verbose, write_text_file_verbose, ask_yes_no, ask_for_input, \
    get_user_email_git, email_is_valid
from ..utils.wizard import Wizard

from snipsskillscore.logging import log, log_success, log_error


# pylint: disable=too-few-public-methods
class Scaffold(Base):
    """
    The scaffold command.

    The goal is to generate the following structure for a Snips skill

    /[projectname]/
    /[projectname]/setup.py
    /[projectname]/README.rst
    /[projectname]/LICENSE.txt
    /[projectname]/MANIFEST.in
    /[projectname]/lint.cfg
    /[projectname]/setup.cfg
    /[projectname]/[projectname]
    /[projectname]/[projectname]/__init__.py
    /[projectname]/[projectname]/Snipsspec
    /[projectname]/tests
    /[projectname]/tests/__init__.py
    /[projectname]/tests/[projectname]_tests.py
    """

    def __init__(self):
        self.jinja_env = Environment(
            loader=PackageLoader('snipsskills', 'templates'))

        self.wizard = Wizard()
        self.wizard.add_question(
            description="Give your skill a name. For instance: lightskill, gardeningskill, etc ...",
            text="Project name? ",
            input_function=ask_for_input,
            input_validation=lambda x: len(x) > 0)
        self.wizard.add_question(description="A short sentence to describe what your skill does.",
                                 text="Description? ",
                                 input_function=ask_for_input,
                                 input_validation=lambda x: len(x) > 0)
        self.wizard.add_question(description="",
                                 text="Author? ",
                                 input_function=ask_for_input,
                                 input_validation=lambda x: True)
        self.wizard.add_question(description="",
                                 text="Email address? ",
                                 input_function=ask_for_input,
                                 input_validation=email_is_valid,
                                 default_value=get_user_email_git())

    def run(self):
        current_directory = os.getcwd()

        project_name, description, author, email = [question.answer() for question in self.wizard]
        print "\n"

        try:
            log("Scaffolding {} structure".format(project_name))
            self.create_folders(current_directory, project_name)
            self.create_files(current_directory=current_directory, project_name=project_name, description=description,
                              author=author, email=email)

        except IOError as e:
            log_error(e.strerror)

    def retrieve_user_information(self):
        """
        The templates need the following variables from the user :
            - project_name
            - pypi identifier
            - description
            - author
            - email
            - github_url
        """

    def create_folders(self, current_directory, project_name):
        root_directory = os.path.join(current_directory, project_name)

        if os.path.exists(root_directory) and os.listdir(root_directory):
            raise IOError()
        else:
            create_dir_verbose(root_directory, 0)

        directory_names = (project_name, 'tests')

        for directory in directory_names:
            directory_name = os.path.join(root_directory, directory)
            create_dir_verbose(directory_name, 1)

    def create_files(self, current_directory, project_name, description, author, email):
        root_directory = os.path.join(current_directory, project_name)

        self.write_setup(root_directory, project_name, email, description, author)
        self.write_snips_spec(project_name, root_directory)
        self.write_inits(project_name, root_directory)
        self.write_unit_tests(project_name, root_directory)
        self.write_readme(project_name, root_directory)
        self.write_manifest(project_name, root_directory)
        self.write_configs(project_name, root_directory)

    def write_setup(self, root_directory, project_name, email, description, author):
        setup_path = os.path.join(root_directory, 'setup.py')

        SETUP_template = self.jinja_env.get_template('setup.py')
        setup_content = SETUP_template.render(project_name=project_name, email=email, description=description,
                                              author=author)
        write_text_file_verbose(setup_path, setup_content, 1)

    def write_snips_spec(self, project_name, root_directory):
        spec_path = os.path.join(root_directory, project_name, 'Snipsspec')

        spec_content = ''
        write_text_file_verbose(spec_path, spec_content, 2)

    def write_inits(self, project_name, root_directory):
        project_init = os.path.join(root_directory, project_name, '__init__.py')
        tests_init = os.path.join(root_directory, 'tests', '__init__.py')

        write_text_file_verbose(project_init, '', 2)
        write_text_file_verbose(tests_init, '', 2)

    def write_unit_tests(self, project_name, root_directory):
        pass

    def write_readme(self, project_name, root_directory):
        readme_path = os.path.join(root_directory, 'README.rst')

        README_template = self.jinja_env.get_template('README.rst')
        README_content = README_template.render(project_name=project_name)
        write_text_file_verbose(readme_path, README_content, 1)

    def write_license(self, project_name, root_directory):
        license_path = os.path.join(root_directory, 'LICENSE.txt')
        LICENSE_template = self.jinja_env.get_template('LICENSE.txt')

        write_text_file_verbose(license_path, LICENSE_template.render(), 1)

    def write_manifest(self, project_name, root_directory):
        manifest_path = os.path.join(root_directory, 'MANIFEST.in')

        MANIFEST_template = self.jinja_env.get_template('MANIFEST.in')
        manifest_content = MANIFEST_template.render(project_name=project_name)
        write_text_file_verbose(manifest_path, manifest_content, 1)

    def write_configs(self, project_name, root_directory):
        lint_path = os.path.join(root_directory, 'lint.cfg')
        setup_path = os.path.join(root_directory, 'setup.cfg')

        LINT_template = self.jinja_env.get_template('lint.cfg')
        SETUP_CONFIG_template = self.jinja_env.get_template('setup.cfg')

        lint_content = LINT_template.render()
        setup_content = SETUP_CONFIG_template.render()

        write_text_file_verbose(lint_path, lint_content, 1)
        write_text_file_verbose(setup_path, setup_content, 1)



# -*-: coding utf-8 -*-
""" Snips Daemon for controlling Snips. """

import json
import time

from socket import error as socket_error

import paho.mqtt.client as mqtt

from snipsskillscore.logging import debug_log, LOGGING_ENABLED
from snipsskillscore.thread_handler import ThreadHandler

from .os_helpers import reboot, get_sysinfo

class RPCDaemon():
    """ Snips Daemon for controlling Snips. """

    def __init__(self,
                 logging_enabled):
        """ Initialisation.

        """
        self.thread_handler = ThreadHandler()

        LOGGING_ENABLED = logging_enabled

    def start(self):
        """ Start the MQTT client. """
        self.thread_handler.run(target=self.start_blocking)
        self.thread_handler.start_run_loop()

    def start_blocking(self, run_event):
        """ Start the BLE client, as a blocking method.

        :param run_event: a run event object provided by the thread handler.
        """
        pass

    # pylint: disable=unused-argument
    def on_message(self, client, userdata, msg):
        """ Callback when the BLE client received a new message.

        :param client: the BLE client.
        :param userdata: unused.
        :param msg: the BLE message.
        """
        if msg.topic == "daemon/reboot":
            self.handle_reboot()
        elif msg.topic == "daemon/ask_sysinfo":
            self.handle_ask_sysinfo()
        	# self.handle_reboot()
            # payload = json.loads(msg.payload.decode('utf-8'))
            # intent = IntentParser.parse(payload, self.registry.intent_classes)
            # debug_log("New intent: {}".format(str(intent.intentName)))
            # if self.handle_intent:
            #     self.handle_intent(intent)

    def publish(self, topic, message):
        """ Publish a message to BLE.

        :param topic: the topic on which to publish the message.
        :param message: the message to publish.
        """
        pass

    def handle_reboot(self):
        """ Handle a reboot message."""
        reboot()

    def handle_ask_sysinfo(self):
        """ Handle an ask_sysinfo message."""

        self.publish("daemon/reply_sysinfo", get_sysinfo())

    def handle_write_snipsfile(self, snipsfile):
        """ Handle a write_snipsfile message."""
        pass

    def handle_install(self):
        """ Handle a snipsskills install message."""
        pass


# -*-: coding utf-8 -*-
""" Downloader for Snips assistants. """

from http_helpers import post_request_json
import os
import json
import re

from ..utils.os_helpers import email_is_valid

try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen, Request, URLError

USER_AUTH_ROUTE = "https://private-gateway.snips.ai/v1/user/auth"


class AuthException(Exception):
    pass


class AuthExceptionInvalidCredentials(AuthException):
    pass


class AuthExceptionInvalidAssistantId(AuthException):
    pass


class DownloaderException(Exception):
    """ Exceptions related to downloads of Snips assistants. """
    pass


# pylint: disable=too-few-public-methods
class Downloader(object):
    """ Downloader for Snips assistants. """

    @staticmethod
    def download(url, output_dir, filename):
        """ Download a file, and save it to a file.

        :param url: the URL of the file.
        :param output_dir: the directory where the file should be
                           saved.
        """
        try:
            response = urlopen(url)
        except Exception:
            raise DownloaderException()

        Downloader.save(response.read(),
                        output_dir,
                        filename)

    @staticmethod
    def save(content, output_dir, filename):
        """ Save content of a file.

        :param content: the content of the file to save.
        :param output_dir: the directory where the assistant should be
                           saved.
        """
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
        output_filename = "{}/{}".format(output_dir, filename)
        with open(output_filename, "wb") as output_file:
            output_file.write(content)


class AuthDownloader(Downloader):
    def __init__(self, email, password, assistantId):
        self.email = email
        self.password = password
        self.assistant_id = assistantId
        self.validate_input()

    @property
    def auth_url(self):
        raise NotImplementedError

    @property
    def download_url(self):
        raise NotImplementedError

    def validate_input(self):
        if not email_is_valid(self.email):
            raise AuthExceptionInvalidCredentials("Error, Email is not valid")

        if len(self.password) < 1:
            raise AuthExceptionInvalidCredentials("Error, password is too short")

        if len(self.assistant_id) < 14:
            raise AuthExceptionInvalidAssistantId("Error, assistantId is too short")

    def retrieve_auth_token(self):
        data = {'email': self.email, 'password': self.password}

        try:
            response, response_headers = post_request_json(self.auth_url, data)
            token = response_headers.getheader('Authorization')
            return token
        except URLError:
            raise DownloaderException

    def download(self, output_dir, filename):
        try:
            token = self.retrieve_auth_token()
            request = Request(self.download_url, headers={'Authorization': token, 'Accept': 'application/json'})
            response = urlopen(request)
        except Exception:
            raise DownloaderException()

        Downloader.save(response.read(),
                        output_dir,
                        filename)


class AssistantDownloader(AuthDownloader):
    auth_url = "https://private-gateway.snips.ai/v1/user/auth"
    download_url = "https://private-gateway.snips.ai/v1/assistant/{}/download"

    def __init__(self, email, password, assistantId):
        AuthDownloader.__init__(self, email, password, assistantId)
        self.download_url = self.download_url.format(self.assistant_id)



class Wizard(object):
    def __init__(self):
        self._questions = list()

    def __len__(self):
        return len(self._questions)

    def __getitem__(self, position):
        return self._questions[position]

    def add_question(self, text, description, input_function, input_validation, default_value=None):
        question = Question(text=text, description=description, input_function=input_function,
                            input_validation=input_validation, default_value=default_value)
        self._questions.append(question)

    def run(self):
        return [question.answer() for question in self._questions]


class Question(object):
    def __init__(self, text, description, input_function, input_validation, default_value=None):
        self.text = text
        self.description = description
        self.input_function = input_function
        self.input_validation = input_validation
        self.default_value = default_value

    def answer(self):
        if len(self.description) > 0:
            print self.description

        result = self.input_function(self.text) if not self.default_value else self.input_function(self.text,
                                                                                                   self.default_value)
        while (not self.input_validation(result)):
            result = self.input_function(self.text) if not self.default_value else self.input_function(self.text,
                                                                                                       self.default_val)
        return result



# -*-: coding utf-8 -*-
""" Systemd utilities. """

import getpass
import os
import subprocess
import time

from .os_helpers import execute_command, ask_yes_no, which

SNIPS_SERVICE_NAME = "snips"
SNIPSSKILLS_SERVICE_NAME = "snipsskills"


class Systemd:
    """ Systemd utilities. """

    @staticmethod
    def setup():
        if ask_yes_no("Would you like Snips to start on boot (using systemd)?") == False:
            return

        (username, snips_home_path, snipsskills_path,
         snips_path) = Systemd.get_snipsskills_params()
        Systemd.write_snips_file(username, snips_path)
        Systemd.write_snipsskills_file(
            username, snips_home_path, snipsskills_path)
        Systemd.enable_service(username, SNIPS_SERVICE_NAME)
        Systemd.enable_service(username, SNIPSSKILLS_SERVICE_NAME)

    @staticmethod
    def get_snipsskills_params():
        current_username = getpass.getuser()
        username = raw_input(
            "Run as user [default: {}]: ".format(current_username))
        if username is None or username.strip() == "":
            username = current_username

        snips_home_path = os.getcwd()

        snipsskills_path = which('snipsskills')
        if snipsskills_path is None or len(snipsskills_path.strip()) == 0:
            snipsskills_path = raw_input("Path to the snipsskills binary: ")

        snips_path = which('snips')
        if snips_path is None or len(snips_path.strip()) == 0:
            snips_path = raw_input("Path to the snips binary: ")

        return (username, snips_home_path, snipsskills_path, snips_path)

    @staticmethod
    def write_snipsskills_file(username, snips_home_path, snipsskills_path):
        contents = Systemd.get_template(SNIPSSKILLS_SERVICE_NAME)
        if contents is None:
            return
        contents = contents.replace("{{SNIPS_HOME_PATH}}", snips_home_path) \
            .replace("{{SNIPSSKILLS_PATH}}", snipsskills_path)
        Systemd.write_systemd_file(
            SNIPSSKILLS_SERVICE_NAME, username, contents)

    @staticmethod
    def write_snips_file(username, snips_path):
        contents = Systemd.get_template(SNIPS_SERVICE_NAME)
        if contents is None:
            return
        contents = contents.replace("{{SNIPS_PATH}}", snips_path)
        Systemd.write_systemd_file(SNIPS_SERVICE_NAME, username, contents)

    @staticmethod
    def get_template(service_name):
        this_dir, this_filename = os.path.split(__file__)
        template_filename = os.path.join(
            this_dir, "../config/systemd/{}.service".format(service_name))
        with open(template_filename, 'r') as template_file:
            return template_file.read()
        return None

    @staticmethod
    def write_systemd_file(service_name, username, contents):
        if username is None:
            # Run as root
            output_filename = "/etc/systemd/system/{}.service".format(
                service_name)
        else:
            output_filename = "/etc/systemd/system/{}@{}.service".format(
                service_name, username)
        tmp_filename = "tmp_{}".format(service_name)
        os.system("sudo echo \"{}\" > {}".format(contents, tmp_filename))
        os.system("sudo mv {} {}".format(tmp_filename, output_filename))
        os.system("sudo chmod a+rwx {}".format(output_filename))

    @staticmethod
    def enable_service(username, service_name):
        os.system("sudo systemctl --system daemon-reload")
        if username is None:
            # Run as root
            os.system("sudo systemctl enable {}".format(service_name))
        else:
            os.system("sudo systemctl enable {}@{}".format(service_name, username))



# -*-: coding utf-8 -*-
""" pip module installer. """

import pip

# pylint: disable=too-few-public-methods
class PipInstaller:
    """ pip module installer. """

    @staticmethod
    def install(module):
        """ Install a pip module.

        :param module: the pip module name or URL.
        """
        pip.main(['install', '--quiet', module])



from setuptools import setup

setup(
    name='',
    version='0.0.1',
    description='{{description}}',
    author='{{author}}',
    author_email='{{email}}',
    download_url='',
    license='MIT',
    install_requires=[],
    setup_requires=['green'],
    keywords=['snips'],
    include_package_data=True,
    packages=[
        '{{project_name}}'
    ]
)


# -*-: coding utf-8 -*-
""" The base command. """

SNIPSFILE = "Snipsfile"
ASSISTANT_DIR = ".snips"
ASSISTANT_ZIP_FILENAME = "assistant.zip"
ASSISTANT_ZIP_PATH = "{}/{}".format(ASSISTANT_DIR, ASSISTANT_ZIP_FILENAME)
INTENTS_DIR = ".snips/intents"

# pylint: disable=too-few-public-methods
class Base(object):
    """ The base command. """

    def __init__(self, options, *args, **kwargs):
        """ Initialisation.

        :param options: command-line options.
        :param *args, **kwargs: extra arguments.
        """
        self.options = options
        self.args = args
        self.kwargs = kwargs
        self.snipsfile = None

    def run(self):
        """ Command runner. """
        raise NotImplementedError(
            'You must implement the run() method yourself!')



import json
import urllib2

def post_request(url, data, headers):
    """
    :param url:
    :type url: basestring
    :param data:
    :type data: dict
    :param headers:
    :type headers: dict
    :return:
    :rtype: basestring
    """
    raw_data = json.dumps(data)
    req = urllib2.Request(url,
                          raw_data,
                          headers)
    f = urllib2.urlopen(req)
    info = f.info()
    response = f.read()
    f.close()
    return response, info

def post_request_json(url, data, headers={}):
    """

    :param url:
    :type url: basestring
    :param data:
    :type data: dict
    :param headers:
    :type headers: dict
    :return:
    :rtype: dict
    """
    headers['Content-Type'] = 'application/json'
    headers['Accept'] = 'application/json'

    response, info = post_request(url, data, headers)
    return json.loads(response), info



"""Packaging settings."""

from setuptools import Command, find_packages, setup
from snipsskills import __version__
from codecs import open
from os.path import abspath, dirname, join
from subprocess import call

this_dir = abspath(dirname(__file__))
with open(join(this_dir, 'README.rst'), encoding='utf-8') as file:
    long_description = file.read()


class RunTests(Command):
    """Run all tests."""
    description = 'run tests'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        """Run all tests!"""
        errno = call(['py.test', '--cov=snipsskills', '--cov-report=term-missing'])
        raise SystemExit(errno)


setup(
    name = 'snipsskills',
    version = __version__,
    description = 'Snips Skills Manager',
    long_description = long_description,
    url = 'https://github.com/snipsco/snipsskills',
    author = 'Snips',
    author_email = 'michael.fester@snips.ai',
    license='MIT',
    keywords = ['cli', 'snips'],
    packages = find_packages(exclude=['docs', 'tests*']),
    install_requires = [
        'docopt',
        'python-dateutil',
        'Jinja2',
        'pyyaml',
        'pip',
        'snipsskillscore'
    ],
    extras_require = {
        'test': ['coverage', 'pytest', 'pytest-cov'],
    },
    entry_points = {
        'console_scripts': [
            'snipsskills=snipsskills.cli:main',
        ],
    },
    include_package_data=True,
    cmdclass = {'test': RunTests}
)



# -*-: coding utf-8 -*-
""" Skill definition from a YAML config. """

# pylint: disable=too-few-public-methods
class SkillDef:
    """ Skill definition from a YAML config. """

    # pylint: disable=too-many-arguments
    def __init__(self, package_name, class_name, pip, params, intent_defs, requires_tts):
        """ Initialisation.

        :param package_name: the name of the Python module.
        :param class_name: the name of the Python class.
        :param pip: the pip package (name or url).
        :param params: the parameters to pass to the skills constructor.
        :param intent_defs: a list of intent definitions.
        :param requires_tts: whether the skill requires TTS.
        """
        self.package_name = package_name
        self.class_name = class_name
        self.pip = pip
        self.params = params
        self.intent_defs = intent_defs
        self.requires_tts = requires_tts

    def find(self, intent):
        """ Find an intent definition in the list of intents that the skill
            declares.

        :param intent: the intent object to look for.
        :return: an intent definition, from the skill definition, if found,
                 or None.
        """
        for intent_def in self.intent_defs:
            if intent_def.name == intent.intentName:
                return intent_def
        return None
